@@ -10,6 +10,7 @@
 #include <config.h>
 #endif
 
+#include <string.h>
 #include <bonobo/bonobo-listener.h>
 
 #include <cddb-slave-client.h>
@@ -33,7 +34,6 @@ count_tracks (CDDBSlaveClientTrackInfo **info)
 		;
 	}
 
-	g_print ("number of tracks: %d\n", i);
 	return i;
 }
 
@@ -55,14 +55,13 @@ get_disc_info (GnomeCD *gcd,
 	info->artist = cddb_slave_client_get_artist (slave, discid);
 	info->track_info = cddb_slave_client_get_tracks (slave, discid);
 
-	g_print ("info->ntracks = %d\n", info->ntracks);
 	if (count_tracks (info->track_info) != info->ntracks) {
 		/* Duff info */
 		cddb_slave_client_free_track_info (info->track_info);
 		info->track_info = NULL;
 		gcd->disc_info = NULL;
 	}
-	
+
 	gnome_cd_build_track_list_menu (gcd);
 }
 
@@ -128,8 +127,8 @@ cddb_make_disc_info (GnomeCDRomCDDBData *data)
 {
 	GnomeCDDiscInfo *discinfo;
 
-	discinfo = g_new (GnomeCDDiscInfo, 1);
-	discinfo->discid = g_strdup_printf ("%08lx", data->discid);
+	discinfo = g_new0 (GnomeCDDiscInfo, 1);
+	discinfo->discid = g_strdup_printf ("%08lx", (gulong) data->discid);
 	discinfo->title = NULL;
 	discinfo->artist = NULL;
 	discinfo->ntracks = data->ntrks;
@@ -139,12 +138,12 @@ cddb_make_disc_info (GnomeCDRomCDDBData *data)
 }
 
 void
-close_cddb_client (void)
+cddb_close_client (void)
 {
-	if (slave != NULL) {
+	if (slave != NULL)
 		g_object_unref (G_OBJECT (slave));
-	}
 }
+
 void
 cddb_get_query (GnomeCD *gcd)
 {
@@ -157,7 +156,6 @@ cddb_get_query (GnomeCD *gcd)
 
 	if (cddb_cache == NULL) {
 		cddb_cache = g_hash_table_new (g_str_hash, g_str_equal);
-		/* cddb_preload_cache (); */
 	}
 
 	if (gnome_cdrom_get_cddb_data (gcd->cdrom, &data, NULL) == FALSE) {
@@ -165,7 +163,7 @@ cddb_get_query (GnomeCD *gcd)
 		return;
 	}
 
-	discid = g_strdup_printf ("%08lx", data->discid);
+	discid = g_strdup_printf ("%08lx", (gulong) data->discid);
 	for (i = 0; i < data->ntrks; i++) {
 		char *tmp;
 
