@@ -11,6 +11,7 @@
 #include <config.h>
 #endif
 
+#include <libgnome/gnome-i18n.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
@@ -34,11 +35,6 @@ typedef struct _FreeBSDCDRomTrackInfo {
 } FreeBSDCDRomTrackInfo;
 
 struct _FreeBSDCDRomPrivate {
-	char *cdrom_device;
-	guint32 update_id;
-	int cdrom_fd;
-	int ref_count;
-
 	GnomeCDRomUpdate update;
 
 	struct ioc_toc_header *tochdr;
@@ -46,8 +42,6 @@ struct _FreeBSDCDRomPrivate {
 	unsigned char track0, track1;
 
 	FreeBSDCDRomTrackInfo *track_info;
-	
-	GnomeCDRomStatus *recent_status;
 };
 
 static gboolean freebsd_cdrom_eject (GnomeCDRom *cdrom,
@@ -79,20 +73,11 @@ static gboolean freebsd_cdrom_close_tray (GnomeCDRom *cdrom,
 static GnomeCDRomMSF blank_msf = { 0, 0, 0};
 
 static void
-finalize (GObject *object)
+freebsd_cdrom_finalize (GObject *object)
 {
-	FreeBSDCDRom *cdrom;
-
-	cdrom = FREEBSD_CDROM (object);
-	if (cdrom->priv == NULL) {
-		return;
-	}
-
-	close (cdrom->priv->cdrom_fd);
-	g_free (cdrom->priv->cdrom_device);
+	FreeBSDCDrom *cdrom = (FreeBSDCDrom *) object;
 
 	g_free (cdrom->priv);
-	cdrom->priv = NULL;
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -133,47 +118,15 @@ add_msf (GnomeCDRomMSF *msf1,
 
 static gboolean
 freebsd_cdrom_open (FreeBSDCDRom *lcd,
-		  GError **error)
+		    GError      **error)
 {
-	if (lcd->priv->cdrom_fd != -1) {
-		lcd->priv->ref_count++;
-		return TRUE;
-	}
-
-	lcd->priv->cdrom_fd = open (lcd->priv->cdrom_device, O_RDONLY | O_NONBLOCK);
-	if (lcd->priv->cdrom_fd < 0) {
-		if (error) {
-			*error = g_error_new (GNOME_CDROM_ERROR,
-					      GNOME_CDROM_ERROR_NOT_OPENED,
-					      "Unable to open %s. This may be caused by:\n"
-					      "a) CD support is not compiled into the FreeBSD kernel\n"
-					      "b) You do not have the correct permissions to access the CD drive\n"
-					      "c) %s is not the CD drive.\n",					     
-					      lcd->priv->cdrom_device, lcd->priv->cdrom_device);
-		}
-
-		return FALSE;
-	}
-
-	lcd->priv->ref_count = 1;
-	return TRUE;
+	return gnome_cdrom_open_dev (GNOME_CDROM (lcd), error);
 }
 
 static void
 freebsd_cdrom_close (FreeBSDCDRom *lcd)
 {
-	if (lcd->priv->cdrom_fd == -1) {
-		return;
-	}
-
-	lcd->priv->ref_count--;
-
-	if (lcd->priv->ref_count <= 0) {
-		close (lcd->priv->cdrom_fd);
-		lcd->priv->cdrom_fd = -1;
-	}
-
-	return;
+	gnome_cdrom_close_dev (GNOME_CDROM (lcd), FALSE);
 }
 
 static void
@@ -209,8 +162,9 @@ calculate_track_lengths (FreeBSDCDRom *lcd)
 }
 
 static void
-freebsd_cdrom_update_cd (FreeBSDCDRom *lcd)
+freebsd_cdrom_update_cd (GnomeCDRom *cdrom)
 {
+	FreeBSDCDRom *lcd = FREEBSD_CDROM (cdrom);
 	FreeBSDCDRomPrivate *priv;
 	struct ioc_read_toc_single_entry tocentry;
 	int i, j;
@@ -219,12 +173,12 @@ freebsd_cdrom_update_cd (FreeBSDCDRom *lcd)
 	priv = lcd->priv;
 
 	if (freebsd_cdrom_open (lcd, &error) == FALSE) {
-		g_warning ("Error opening CD");
+		g_warning (_("Error opening CD"));
 		return;
 	}
 
-	if (ioctl (priv->cdrom_fd, CDIOREADTOCHEADER, priv->tochdr) < 0) {
-		g_warning ("Error reading CD header");
+	if (ioctl (cdrom->fd, CDIOREADTOCHEADER, priv->tochdr) < 0) {
+		g_warning (_("Error reading CD header"));
 		freebsd_cdrom_close (lcd);
 
 		return;
@@ -240,8 +194,8 @@ freebsd_cdrom_update_cd (FreeBSDCDRom *lcd)
 		tocentry.track = j;
 		tocentry.address_format = CD_MSF_FORMAT;
 
-		if (ioctl (priv->cdrom_fd, CDIOREADTOCENTRY, &tocentry) < 0) {
-			g_warning ("IOCtl failed");
+		if (ioctl (cdrom->fd, CDIOREADTOCENTRY, &tocentry) < 0) {
+			g_warning (_("IOCtl failed"));
 			continue;
 		}
 
@@ -253,8 +207,8 @@ freebsd_cdrom_update_cd (FreeBSDCDRom *lcd)
 	/* On FreeBSD, the leadout track is the track LAST_TRACK + 1. */
 	tocentry.track = priv->number_tracks + 1;
 	tocentry.address_format = CD_MSF_FORMAT;
-	if (ioctl (priv->cdrom_fd, CDIOREADTOCENTRY, &tocentry) < 0) {
-		g_warning ("Error getting leadout");
+	if (ioctl (cdrom->fd, CDIOREADTOCENTRY, &tocentry) < 0) {
+		g_warning (_("Error getting leadout"));
 		freebsd_cdrom_invalidate (lcd);
 		return;
 	}
@@ -262,8 +216,8 @@ freebsd_cdrom_update_cd (FreeBSDCDRom *lcd)
 	calculate_track_lengths (lcd);
 
 #ifdef DEBUG
-	g_print ("CD changed\n"
-		 "Track count: %d\n------------------\n",
+	g_print (_("CD changed\n"
+		 "Track count: %d\n------------------\n"),
 		 priv->number_tracks);
 #endif
 	
@@ -273,7 +227,7 @@ freebsd_cdrom_update_cd (FreeBSDCDRom *lcd)
 
 static gboolean
 freebsd_cdrom_eject (GnomeCDRom *cdrom,
-		   GError **error)
+		     GError    **error)
 {
 	FreeBSDCDRom *lcd;
 	GnomeCDRomStatus *status;
@@ -290,11 +244,11 @@ freebsd_cdrom_eject (GnomeCDRom *cdrom,
 	}
 
 	if (status->cd != GNOME_CDROM_STATUS_TRAY_OPEN) {
-		if (ioctl (lcd->priv->cdrom_fd, CDIOCEJECT, 0) < 0) {
+		if (ioctl (cdrom->fd, CDIOCEJECT, 0) < 0) {
 			if (error) {
 				*error = g_error_new (GNOME_CDROM_ERROR,
 						      GNOME_CDROM_ERROR_SYSTEM_ERROR,
-						      "(eject): ioctl failed: %s",
+						      _("(eject): ioctl failed: %s"),
 						      strerror (errno));
 			}
 
@@ -314,7 +268,9 @@ freebsd_cdrom_eject (GnomeCDRom *cdrom,
 	}
 
 	g_free (status);
-	freebsd_cdrom_close (lcd);
+
+	gnome_cdrom_close_dev (cdrom, TRUE);
+
 	return TRUE;
 }
 
@@ -392,6 +348,13 @@ freebsd_cdrom_ffwd (GnomeCDRom *cdrom,
 		return FALSE;
 	}
 
+	if (status->cd != GNOME_CDROM_STATUS_OK) {
+		freebsd_cdrom_close (lcd);
+		g_free (status);
+		
+		return TRUE;
+	}
+
 	msf = &status->absolute;
 	/* Convert MSF to frames to do calculations on it */
 	frames = msf_to_frames (msf);
@@ -471,7 +434,7 @@ freebsd_cdrom_play (GnomeCDRom *cdrom,
 			if (error) {
 				*error = g_error_new (GNOME_CDROM_ERROR,
 						      GNOME_CDROM_ERROR_NOT_READY,
-						      "(freebsd_cdrom_play): Drive not ready");
+						      _("(freebsd_cdrom_play): Drive not ready"));
 			}
 
 			freebsd_cdrom_close (lcd);
@@ -491,7 +454,7 @@ freebsd_cdrom_play (GnomeCDRom *cdrom,
 		if (error) {
 			*error = g_error_new (GNOME_CDROM_ERROR,
 					      GNOME_CDROM_ERROR_NOT_READY,
-					      "(freebsd_cdrom_play): Drive still not ready");
+					      _("(freebsd_cdrom_play): Drive still not ready"));
 		}
 
 		freebsd_cdrom_close (lcd);
@@ -552,11 +515,11 @@ freebsd_cdrom_play (GnomeCDRom *cdrom,
 		}
 
 		/* PLAY IT AGAIN */
-		if (ioctl (priv->cdrom_fd, CDIOCPLAYMSF, &msf) < 0) {
+		if (ioctl (cdrom->fd, CDIOCPLAYMSF, &msf) < 0) {
 			if (error) {
 				*error = g_error_new (GNOME_CDROM_ERROR,
 						      GNOME_CDROM_ERROR_SYSTEM_ERROR,
-						      "(freebsd_cdrom_play) ioctl failed %s",
+						      _("(freebsd_cdrom_play) ioctl failed %s"),
 						      strerror (errno));
 			}
 
@@ -592,7 +555,7 @@ freebsd_cdrom_pause (GnomeCDRom *cdrom,
 		if (error) {
 			*error = g_error_new (GNOME_CDROM_ERROR,
 					      GNOME_CDROM_ERROR_NOT_READY,
-					      "(freebsd_cdrom_pause): Drive not ready");
+					      _("(freebsd_cdrom_pause): Drive not ready"));
 		}
 
 		g_free (status);
@@ -601,11 +564,11 @@ freebsd_cdrom_pause (GnomeCDRom *cdrom,
 	}
 
 	if (status->audio == GNOME_CDROM_AUDIO_PAUSE) {
-		if (ioctl (lcd->priv->cdrom_fd, CDIOCRESUME) < 0) {
+		if (ioctl (cdrom->fd, CDIOCRESUME) < 0) {
 			if (error) {
 				*error = g_error_new (GNOME_CDROM_ERROR,
 						      GNOME_CDROM_ERROR_SYSTEM_ERROR,
-						      "(freebsd_cdrom_pause): Resume failed %s",
+						      _("(freebsd_cdrom_pause): Resume failed %s"),
 						      strerror (errno));
 			}
 
@@ -620,11 +583,11 @@ freebsd_cdrom_pause (GnomeCDRom *cdrom,
 	}
 
 	if (status->audio == GNOME_CDROM_AUDIO_PLAY) {
-		if (ioctl (lcd->priv->cdrom_fd, CDIOCPAUSE, 0) < 0) {
+		if (ioctl (cdrom->fd, CDIOCPAUSE, 0) < 0) {
 			if (error) {
 				*error = g_error_new (GNOME_CDROM_ERROR,
 						      GNOME_CDROM_ERROR_SYSTEM_ERROR,
-						      "(freebsd_cdrom_pause): ioctl failed %s",
+						      _("(freebsd_cdrom_pause): ioctl failed %s"),
 						      strerror (errno));
 			}
 
@@ -666,11 +629,11 @@ freebsd_cdrom_stop (GnomeCDRom *cdrom,
 	}
 #endif
 
-	if (ioctl (lcd->priv->cdrom_fd, CDIOCSTOP, 0) < 0) {
+	if (ioctl (cdrom->fd, CDIOCSTOP, 0) < 0) {
 		if (error) {
 			*error = g_error_new (GNOME_CDROM_ERROR,
 					      GNOME_CDROM_ERROR_SYSTEM_ERROR,
-					      "(freebsd_cdrom_stop) ioctl failed %s",
+					      _("(freebsd_cdrom_stop) ioctl failed %s"),
 					      strerror (errno));
 		}
 
@@ -826,6 +789,7 @@ freebsd_cdrom_get_status (GnomeCDRom *cdrom,
 	GnomeCDRomStatus *realstatus;
 	struct ioc_read_subchannel subchnl;
 	struct cd_sub_channel_info subchnl_info;
+	struct ioc_vol vol;
 	int cd_status;
 
 	g_return_val_if_fail (status != NULL, TRUE);
@@ -833,8 +797,9 @@ freebsd_cdrom_get_status (GnomeCDRom *cdrom,
 	lcd = FREEBSD_CDROM (cdrom);
 	priv = lcd->priv;
 
-	*status = g_new (GnomeCDRomStatus, 1);
+	*status = g_new0 (GnomeCDRomStatus, 1);
 	realstatus = *status;
+	realstatus->volume = 0;
 
 	if (freebsd_cdrom_open (lcd, error) == FALSE) {
 		g_free (realstatus);
@@ -843,7 +808,7 @@ freebsd_cdrom_get_status (GnomeCDRom *cdrom,
 	}
 
 #if 0
-	cd_status = ioctl (priv->cdrom_fd, CDROM_DRIVE_STATUS, CDSL_CURRENT);
+	cd_status = ioctl (cdrom->fd, CDROM_DRIVE_STATUS, CDSL_CURRENT);
 	if (cd_status != -1) {
 		switch (cd_status) {
 		case CDS_NO_INFO:
@@ -886,7 +851,7 @@ freebsd_cdrom_get_status (GnomeCDRom *cdrom,
 		if (error) {
 			*error = g_error_new (GNOME_CDROM_ERROR,
 					      GNOME_CDROM_ERROR_SYSTEM_ERROR,
-					      "(freebsd_cdrom_get_status): ioctl error %s",
+					      _("(freebsd_cdrom_get_status): ioctl error %s"),
 					      strerror (errno));
 		}
 
@@ -902,11 +867,11 @@ freebsd_cdrom_get_status (GnomeCDRom *cdrom,
 	subchnl.data_format = CD_CURRENT_POSITION;
 	subchnl.data = &subchnl_info;
 	subchnl.data_len = sizeof(struct cd_sub_channel_info);
-	if (ioctl (priv->cdrom_fd, CDIOCREADSUBCHANNEL, &subchnl) < 0) {
+	if (ioctl (cdrom->fd, CDIOCREADSUBCHANNEL, &subchnl) < 0) {
 		if (error) {
 			*error = g_error_new (GNOME_CDROM_ERROR,
 					      GNOME_CDROM_ERROR_SYSTEM_ERROR,
-					      "(freebsd_cdrom_get_status): CDIOCREADSUBCHANNEL ioctl failed %s",
+					      _("(freebsd_cdrom_get_status): CDIOCREADSUBCHANNEL ioctl failed %s"),
 					      strerror (errno));
 		}
 
@@ -915,6 +880,15 @@ freebsd_cdrom_get_status (GnomeCDRom *cdrom,
 		return FALSE;
 	}
 
+	/* Get the volume */
+	if (ioctl (cdrom->fd, CDIOCGETVOL, &vol) < 0) {
+		g_warning (_("(freebsd_cdrom_get_status): CDIOCGETVOL ioctl failed %s"),
+			   strerror (errno));
+		realstatus->volume = -1; /* -1 means no volume command */
+	} else {
+		realstatus->volume = vol.vol[0];
+	}
+
 	freebsd_cdrom_close (lcd);
 
 	ASSIGN_MSF (realstatus->relative, blank_msf);
@@ -925,21 +899,21 @@ freebsd_cdrom_get_status (GnomeCDRom *cdrom,
 		realstatus->audio = GNOME_CDROM_AUDIO_PLAY;
 		ASSIGN_MSF (realstatus->relative, subchnl.data->what.position.reladdr.msf);
 		ASSIGN_MSF (realstatus->absolute, subchnl.data->what.position.absaddr.msf);
-		realstatus->track = subchnl.track;
+		realstatus->track = subchnl.data->what.position.track_number;
 		break;
 
 	case CD_AS_PLAY_PAUSED:
 		realstatus->audio = GNOME_CDROM_AUDIO_PAUSE;
 		ASSIGN_MSF (realstatus->relative, subchnl.data->what.position.reladdr.msf);
 		ASSIGN_MSF (realstatus->absolute, subchnl.data->what.position.absaddr.msf);
-		realstatus->track = subchnl.track;
+		realstatus->track = subchnl.data->what.position.track_number;
 		break;
 
 	case CD_AS_PLAY_COMPLETED:
 		realstatus->audio = GNOME_CDROM_AUDIO_COMPLETE;
 		ASSIGN_MSF (realstatus->relative, subchnl.data->what.position.reladdr.msf);
 		ASSIGN_MSF (realstatus->absolute, subchnl.data->what.position.absaddr.msf);
-		realstatus->track = subchnl.track;
+		realstatus->track = subchnl.data->what.position.track_number;
 		break;
 
 	case CD_AS_AUDIO_INVALID:
@@ -967,11 +941,11 @@ freebsd_cdrom_close_tray (GnomeCDRom *cdrom,
 		return FALSE;
 	}
 
-	if (ioctl (lcd->priv->cdrom_fd, CDIOCCLOSE) < 0) {
+	if (ioctl (cdrom->fd, CDIOCCLOSE) < 0) {
 		if (error) {
 			*error = g_error_new (GNOME_CDROM_ERROR,
 					      GNOME_CDROM_ERROR_SYSTEM_ERROR,
-					      "(freebsd_cdrom_close_tray): ioctl failed %s",
+					      _("(freebsd_cdrom_close_tray): ioctl failed %s"),
 					      strerror (errno));
 		}
 
@@ -1002,11 +976,11 @@ freebsd_cdrom_set_volume (GnomeCDRom *cdrom,
 	vol.vol[0] = (u_char)volume;
 	vol.vol[1] = vol.vol[2] = vol.vol[3] = vol.vol[0];
 	
-	if (ioctl (priv->cdrom_fd, CDIOCSETVOL, &vol) < 0) {
+	if (ioctl (cdrom->fd, CDIOCSETVOL, &vol) < 0) {
 		if (error) {
 			*error = g_error_new (GNOME_CDROM_ERROR,
 					      GNOME_CDROM_ERROR_SYSTEM_ERROR,
-					      "(freebsd_cdrom_set_volume:1): ioctl failed %s",
+					      _("(freebsd_cdrom_set_volume:1): ioctl failed %s"),
 					      strerror (errno));
 		}
 
@@ -1023,6 +997,7 @@ freebsd_cdrom_is_cdrom_device (GnomeCDRom *cdrom,
 			       const char *device,
 			       GError **error)
 {
+	struct ioc_vol vol;
 	int fd;
 
 	if (device == NULL || *device == 0) {
@@ -1035,15 +1010,16 @@ freebsd_cdrom_is_cdrom_device (GnomeCDRom *cdrom,
 	}
 
 	/* Fire a harmless ioctl at the device. */
-	if (ioctl (fd, CDIOCSTOP, 0) < 0) {
+	if (ioctl (fd, CDIOCGETVOL, &vol) < 0) {
 		/* Failed, it's not a CDROM drive */
-		g_print ("%s is not a CDROM drive", device);
+		g_print (_("%s is not a CDROM drive\n"), device);
+		g_print ("Strerror reports: %s\n", strerror(errno));
 		close (fd);
 		
 		return FALSE;
 	}
 	
-	g_print ("%s is a CDROM drive", device);
+	g_print (_("%s is a CDROM drive\n"), device);
 	close (fd);
 
 	return TRUE;
@@ -1070,7 +1046,7 @@ freebsd_cdrom_get_cddb_data (GnomeCDRom *cdrom,
 		return TRUE;
 	}
 	
-	*data = g_new (GnomeCDRomCDDBData, 1);
+	*data = g_new0 (GnomeCDRomCDDBData, 1);
 
 	for (i = 0; i < priv->number_tracks; i++) {
 		n += cddb_sum ((priv->track_info[i].address.minute * 60) + 
@@ -1084,7 +1060,7 @@ freebsd_cdrom_get_cddb_data (GnomeCDRom *cdrom,
 	(*data)->discid = ((n % 0xff) << 24 | t << 8 | (priv->track1));
 	(*data)->ntrks = priv->track1;
 	(*data)->nsecs = (priv->track_info[priv->track1].address.minute * 60) + priv->track_info[priv->track1].address.second;
-	(*data)->offsets = g_new (unsigned int, priv->track1 + 1);
+	(*data)->offsets = g_new0 (unsigned int, priv->track1 + 1);
 
 	for (i = priv->track0 - 1; i < priv->track1; i++) {
 		(*data)->offsets[i] = msf_to_frames (&priv->track_info[i].address);
@@ -1094,56 +1070,6 @@ freebsd_cdrom_get_cddb_data (GnomeCDRom *cdrom,
 	return TRUE;
 }
 
-static gboolean
-freebsd_cdrom_set_device (GnomeCDRom *cdrom,
-			const char *device,
-			GError **error)
-{
-	FreeBSDCDRom *lcd;
-	FreeBSDCDRomPrivate *priv;
-	GnomeCDRomStatus *status;
-	
-	lcd = FREEBSD_CDROM (cdrom);
-	priv = lcd->priv;
-
-	if (strcmp (priv->cdrom_device, device) == 0) {
-		/* device is the same */
-		return TRUE;
-	}
-	
-	if (freebsd_cdrom_get_status (cdrom, &status, NULL) == TRUE) {
-		if (status->audio == GNOME_CDROM_AUDIO_PLAY) {
-			if (freebsd_cdrom_stop (cdrom, error) == FALSE) {
-				return FALSE;
-			}
-		}
-	}
-
-	if (priv->cdrom_device != NULL) {
-		g_free (priv->cdrom_device);
-	}
-	priv->cdrom_device = g_strdup (device);
-	
-	if (gnome_cdrom_is_cdrom_device (GNOME_CDROM (lcd),
-					 priv->cdrom_device, error) == FALSE) {
-		if (error) {
-			*error = g_error_new (GNOME_CDROM_ERROR,
-					      GNOME_CDROM_ERROR_NOT_OPENED,
-					      "%s does not point to a valid CDRom device. This may be caused by:\n"
-					      "a) CD support is not compiled into Linux\n"
-					      "b) You do not have the correct permissions to access the CD drive\n"
-					      "c) %s is not the CD drive.\n",					     
-					      priv->cdrom_device, priv->cdrom_device);
-		}
-	}
-
-	/* Force the new CD to be scanned at next update cycle */
-	g_free (priv->recent_status);
-	priv->recent_status = NULL;
-
-	return TRUE;
-}
-
 static void
 class_init (FreeBSDCDRomClass *klass)
 {
@@ -1153,7 +1079,7 @@ class_init (FreeBSDCDRomClass *klass)
 	object_class = G_OBJECT_CLASS (klass);
 	cdrom_class = GNOME_CDROM_CLASS (klass);
 
-	object_class->finalize = finalize;
+	object_class->finalize = freebsd_cdrom_finalize;
 
 	cdrom_class->eject = freebsd_cdrom_eject;
 	cdrom_class->next = freebsd_cdrom_next;
@@ -1167,11 +1093,10 @@ class_init (FreeBSDCDRomClass *klass)
 	cdrom_class->close_tray = freebsd_cdrom_close_tray;
 	cdrom_class->set_volume = freebsd_cdrom_set_volume;
 	cdrom_class->is_cdrom_device = freebsd_cdrom_is_cdrom_device;
+	cdrom_class->update_cd = freebsd_cdrom_update_cd;
 	
 	/* For CDDB */
   	cdrom_class->get_cddb_data = freebsd_cdrom_get_cddb_data;
-
-	cdrom_class->set_device = freebsd_cdrom_set_device;
 	
 	parent_class = g_type_class_peek_parent (klass);
 }
@@ -1179,66 +1104,10 @@ class_init (FreeBSDCDRomClass *klass)
 static void
 init (FreeBSDCDRom *cdrom)
 {
-	cdrom->priv = g_new (FreeBSDCDRomPrivate, 1);
+	cdrom->priv = g_new0 (FreeBSDCDRomPrivate, 1);
 	cdrom->priv->tochdr = g_new (struct ioc_toc_header, 1);
-	cdrom->priv->cdrom_fd = -1;
-	cdrom->priv->ref_count = 0;
-	cdrom->priv->update_id = -1;
-	cdrom->priv->recent_status = NULL;
 	cdrom->priv->track_info = NULL;
 }
-
-static gboolean
-update_cd (gpointer data)
-{
-	FreeBSDCDRom *lcd;
-	FreeBSDCDRomPrivate *priv;
-	GnomeCDRomStatus *status;
-	GError *error;
-
-	lcd = data;
-	priv = lcd->priv;
-
-	/* Do an update */
-	if (freebsd_cdrom_get_status (GNOME_CDROM (lcd), &status, &error) == FALSE) {
-		g_warning ("%s: %s", __FUNCTION__, error->message);
-		
-		g_error_free (error);
-		return TRUE;
-	}
-	
-	if (priv->recent_status == NULL) {
-		priv->recent_status = status;
-		
-		/* emit changed signal */
-		freebsd_cdrom_update_cd (lcd);
-		if (priv->update != GNOME_CDROM_UPDATE_NEVER) {
-			gnome_cdrom_status_changed (GNOME_CDROM (lcd), priv->recent_status);
-		}
-	} else {
-		if (gnome_cdrom_status_equal (status, priv->recent_status) == TRUE) {
-			if (priv->update == GNOME_CDROM_UPDATE_CONTINOUS) {
-				/* emit changed signal */
-				gnome_cdrom_status_changed (GNOME_CDROM (lcd), priv->recent_status);
-			}
-		} else {
-			if (priv->recent_status->cd != GNOME_CDROM_STATUS_OK &&
-			    status->cd == GNOME_CDROM_STATUS_OK) {
-				freebsd_cdrom_update_cd (lcd);
-			}
-			
-			g_free (priv->recent_status);
-			priv->recent_status = status;
-
-			if (priv->update != GNOME_CDROM_UPDATE_NEVER) {
-				/* emit changed signal */
-				gnome_cdrom_status_changed (GNOME_CDROM (lcd), priv->recent_status);
-			}
-		}
-	}
-
-	return TRUE;
-}
 	
 /* API */
 GType
@@ -1259,56 +1128,12 @@ freebsd_cdrom_get_type (void)
 	return type;
 }
 
-/* This is the creation function. It returns a GnomeCDRom object instead of
-   a FreeBSDCDRom one because the caller doesn't know about FreeBSDCDRom only
-   GnomeCDRom */
 GnomeCDRom *
 gnome_cdrom_new (const char *cdrom_device,
 		 GnomeCDRomUpdate update,
 		 GError **error)
 {
-	FreeBSDCDRom *cdrom;
-	FreeBSDCDRomPrivate *priv;
-	int fd;
-
-	cdrom = g_object_new (freebsd_cdrom_get_type (), NULL);
-	priv = cdrom->priv;
-
-	priv->cdrom_device = g_strdup (cdrom_device);
-	priv->update = update;
-
-	freebsd_cdrom_open (cdrom, error);
-	freebsd_cdrom_close (cdrom);
-	if (gnome_cdrom_is_cdrom_device (GNOME_CDROM (cdrom),
-					 cdrom->priv->cdrom_device,
-					 error) == FALSE) {
-		if (error) {
-			*error = g_error_new (GNOME_CDROM_ERROR,
-					      GNOME_CDROM_ERROR_NOT_OPENED,
-					      "%s does not point to a valid CDRom device. This may be caused by:\n"
-					      "a) CD support is not compiled into Linux\n"
-					      "b) You do not have the correct permissions to access the CD drive\n"
-					      "c) %s is not the CD drive.\n",					     
-					      cdrom->priv->cdrom_device, cdrom->priv->cdrom_device);
-		}
-	}
-	
-	/* Force an update so that a status will always exist */
-	update_cd (cdrom);
-	
-	/* All worked, start the counter */
-	switch (update) {
-	case GNOME_CDROM_UPDATE_NEVER:
-		break;
-		
-	case GNOME_CDROM_UPDATE_WHEN_CHANGED:
-	case GNOME_CDROM_UPDATE_CONTINOUS:
-		priv->update_id = g_timeout_add (1000, update_cd, cdrom);
-		break;
-		
-	default:
-		break;
-	}
-
-	return GNOME_CDROM (cdrom);
+	return gnome_cdrom_construct (
+		g_object_new (freebsd_cdrom_get_type (), NULL),
+		cdrom_device, update, error);
 }
